.community-container {
  background-color: rgb(219, 219, 219);
  margin-bottom: 20px;
  margin-top: -28px;
}

.underline {
  border-bottom: 1px solid;
}

.links-underline {
  text-decoration: underline;
}

.community-logo {
  max-width: 100%;
  max-height: 200px;
}

.community-logo-form {
  margin-top: 10px;
  margin-left: 50px;
  max-width: 100%;
  max-height: 200px;
}

.top-padding {
  margin-top: 20px;
}

.title {
  text-align: center;
}

.ui.grid{
  display: block;
  align-items: unset;
  padding: unset;
  margin: auto;
}