{#
  Copyright (C) 2020 CERN.

  Invenio is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}
{% extends "invenio_communities/base.html" %}

{%- block javascript %}
    {{ super() }}
    {{ webpack['invenio-communities-search.js'] }}
{%- endblock %}

{%- block page_body %}
  <div class="ui container top-padding">
    <div class="ui basic segment">
      <a href="/communities/new">
      <button class="ui right floated positive button">
        <i class="fa fa-plus"></i>
        New community
      </button>
    </a>
    </div>
    <div id="communities-search"></div>
  </div>
</div>
{%- endblock page_body %}
