{# -*- coding: utf-8 -*-

  This file is part of Invenio.
  Copyright (C) 2016-2020 CERN.

  Invenio is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}

{% extends "invenio_communities/base.html" %}

{%- block javascript %}
{{ super() }}
{{ webpack['invenio-communities-new.js'] }}
{%- endblock javascript %}

{%- block page_body %}
{% include "invenio_communities/header.html" %}
<div class="container ui grid">
  <div class="two wide column">
    <table class="ui table">
      <thead>
        <tr>
          <th>Community settings</th>
        </tr>
      </thead>
      <tbody>
        <tr>
          <td>Profile</td>
        </tr>
        <tr>
          <td>Collections</td>
        </tr>
        <tr>
          <td>Privileges</td>
        </tr>
        <tr>
          <td>Harvesting</td>
        </tr>
        <tr>
          <td>Webhooks</td>
        </tr>
        <tr>
          <td>Verification</td>
        </tr>
      </tbody>
    </table>
  </div>
  <div class="thirteen wide column">
    <div id="app"></div>
  </div>
  <div id="is_new_community" data-config='{{ False | tojson }}'></div>
</div>
{%- endblock page_body %}
