{# -*- coding: utf-8 -*-

  This file is part of Invenio.
  Copyright (C) 2016-2020 CERN.

  Invenio is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}

{% extends "invenio_communities/base.html" %}

{%- block javascript %}
    {{ super() }}
    {{ webpack['invenio-communities-records-search.js'] }}
{%- endblock %}

{%- block page_body %}
{% include "invenio_communities/header.html" %}
<div class="container top-padding">
  <div id="communities-records-search" data-community-id="{{ community.id }}"></div>
</div>

{%- endblock page_body %}
